@@ -22,12 +22,16 @@
 #define IN_LIBEXSLT
 #include "libexslt/libexslt.h"
 
-#if defined(WIN32) && !defined (__CYGWIN__)
+#if defined(WIN32) && !defined (__CYGWIN__) && (!__MINGW32__)
 #include <win32config.h>
 #else
 #include "config.h"
 #endif
 
+#if HAVE_LOCALTIME_R	/* _POSIX_SOURCE required by gnu libc */
+#define _POSIX_SOURCE
+#endif
+
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
@@ -41,14 +45,21 @@
 
 #include <string.h>
 
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
-
 #ifdef HAVE_MATH_H
 #include <math.h>
 #endif
 
+/* needed to get localtime_r on Solaris */
+#ifdef sun
+#ifndef __EXTENSION__
+#define __EXTENSION__
+#endif
+#endif
+
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
+
 /*
  * types of date and/or time (from schema datatypes)
  *   somewhat ordered from least specific to most specific (i.e.
@@ -77,7 +88,7 @@ struct _exsltDateValDate {
     unsigned int	hour	:5;	/* 0 <=  hour   <= 23   */
     unsigned int	min	:6;	/* 0 <=  min    <= 59	*/
     double		sec;
-    int			tz_flag	:1;	/* is tzo explicitely set? */
+    unsigned int	tz_flag	:1;	/* is tzo explicitely set? */
     int			tzo	:11;	/* -1440 <= tzo <= 1440 */
 };
 
@@ -106,7 +117,8 @@ struct _exsltDateVal {
  *								*
  ****************************************************************/
 
-#if defined(HAVE_TIME_H) && defined(HAVE_LOCALTIME)		\
+#if defined(HAVE_TIME_H) 					\
+    && (defined(HAVE_LOCALTIME) || defined(HAVE_LOCALTIME_R))	\
     && defined(HAVE_TIME) && defined(HAVE_GMTIME)
 #define WITH_TIME
 #endif
@@ -147,8 +159,12 @@ static const unsigned long daysInMonthLeap[12] =
 #define VALID_DATE(dt)						\
 	(VALID_YEAR(dt->year) && VALID_MONTH(dt->mon) && VALID_MDAY(dt))
 
+/*
+    hour and min structure vals are unsigned, so normal macros give
+    warnings on some compilers.
+*/
 #define VALID_TIME(dt)						\
-	(VALID_HOUR(dt->hour) && VALID_MIN(dt->min) &&		\
+	((dt->hour <=23 ) && (dt->min <= 59) &&			\
 	 VALID_SEC(dt->sec) && VALID_TZO(dt->tzo))
 
 #define VALID_DATETIME(dt)					\
@@ -724,6 +740,9 @@ static exsltDateValPtr
 exsltDateCurrent (void) {
     struct tm *localTm, *gmTm;
     time_t secs;
+#if HAVE_LOCALTIME_R
+    struct tm localTmS;
+#endif
     exsltDateValPtr ret;
 
     ret = exsltDateCreateDate(XS_DATETIME);
@@ -732,7 +751,12 @@ exsltDateCurrent (void) {
 
     /* get current time */
     secs    = time(NULL);
+#if HAVE_LOCALTIME_R
+    localtime_r(&secs, &localTmS);
+    localTm = &localTmS;
+#else
     localTm = localtime(&secs);
+#endif
 
     /* get real year, not years since 1900 */
     ret->value.date.year = localTm->tm_year + 1900;
@@ -2875,7 +2899,10 @@ exsltDateDateTimeFunction (xmlXPathParserContextPtr ctxt, int nargs)
     }
 
     ret = exsltDateDateTime();
-    xmlXPathReturnString(ctxt, ret);
+    if (ret == NULL)
+        xmlXPathReturnEmptyString(ctxt);
+    else
+        xmlXPathReturnString(ctxt, ret);
 }
 #endif
 
@@ -3347,7 +3374,10 @@ exsltDateAddFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlFree(ystr);
     xmlFree(xstr);
 
-    xmlXPathReturnString(ctxt, ret);
+    if (ret == NULL)
+        xmlXPathReturnEmptyString(ctxt);
+    else
+	xmlXPathReturnString(ctxt, ret);
 }
 
 /**
@@ -3380,7 +3410,10 @@ exsltDateAddDurationFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlFree(ystr);
     xmlFree(xstr);
 
-    xmlXPathReturnString(ctxt, ret);
+    if (ret == NULL)
+        xmlXPathReturnEmptyString(ctxt);
+    else
+	xmlXPathReturnString(ctxt, ret);
 }
 
 /**
@@ -3413,7 +3446,10 @@ exsltDateDifferenceFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlFree(ystr);
     xmlFree(xstr);
 
-    xmlXPathReturnString(ctxt, ret);
+    if (ret == NULL)
+        xmlXPathReturnEmptyString(ctxt);
+    else
+	xmlXPathReturnString(ctxt, ret);
 }
 
 /**
